{-------------------------------------------------------------------------------
   Title     :  TScnFile
   Version   :  3.00
   Type      :  Shareware Delphi Component
   Cost      :  $10.00 US
   Date      :  August 12, 1996
   Author    :  G. Bradley MacDonald
                TimeAcct Information Systems
                3430 Copeland Avenue
                Vancouver, British Columbia
                Canada
                V5S 4B6
                Email :  Bradley.MacDonald@Deepcove.BC.CA

   Purpose   :  This component is a dual function componet.  It can be used as a
                file find utility - and it can be used as a file scanning utility.
                You tell it the directory, file specifications and file attributes
                to include in the search, as well as whether to include sub
                directories or not.  Then you tell it if you also want to restrict
                the search by file size and file age.  Most important you can tell
                it to only include files that contain a particular string (which you
                can make case sensitive or not).
                Then set the start search property to True and it will create a
                string list with the results of the search.
                So you can use this component to search a file(s) for a string, like
                a grep utility, or as a general seek and scan utility (File Find).
                There should be a small demo project included with this component
                with sample code on how to use it.  (File :  TESTSCAN.PAS)
                See the SCANFILE function for the file scan engine
                See the LOCATEFILE procedure for the file find engine

                New :
                     Version 3.10
                       Now allows you to just pick the component name from a drop
                         downlist.  TaDa!  (Actually it wasn't that hard :))
                         Also added is a Search all available drives function.  This
                         allows the component to search all currently available drives.
                         So instead of forcing you to call each drive sperately - there
                         is now a function that allows you to say "Go find me that file
                         - where ever it is on the drives that are available to me - Now!"
                       Now allows the user to interrupt the search.  See the LOCATEFILE
                         section and the Application.ProcessMessages code.  It is all
                         explained there!
                       Now Includes help file - created by AutoHelp from DinoByte Software

                     Version 3.00
                       Now allows you to update a label on your form with the current
                       search information (ie Currently searching file: XXX) I will
		       be updating this feature in the future to allow you to just
		       pick the component from a drop down list (I Hope!)

                     Version 2.10 (Last Freeware Version)
                       Now includes ability to include file specs in your search
                       (Actually it always did - I just forgot to make the file
                        specs a public property :) Whoops!)
                       Add Ability to show extra info in Search Results.
                          ie File Size, Date, Attributes....
                       Add Ability to append a second scan to the first.
                       Fixed a bug with File Size Scanning.

                     Version 2.00
                       Now works with Delphi 2.0

                     Version 1.10
                       Now you can include filesize and file age in your search
                       criteria.

                NOTE:  It searches all types of files.  It is NOT just a text file
                       scanner.  It uses block IO to scan through all files.

                Limitations:
                1.  The files are read in 30,000 block chunks.  So if the string
                    you are searching for crosses on the 30,000 byte boundry -
                    then the search will not find it.  Highly unlikely - but you
                    never know...
                2.  You should make the search string as small as possible, when
                    searching non-text files.  The problem is with formatted files.
                    For example - word processing documents.  You may have a document
                    that contains the string 'The boy went down the road'.  If you
                    search for the whole string - in a normal text file - you would
                    find it.  But if you are searching a word processing file (or
                    spreadsheet, etc) - then formatting (such as bolding, italic,etc)
                    will add extra characters to the file.  So when the search
                    engine tries to match the string - it won't find a match - because
                    there are extra characters in the string stored in the file.

*** NOTE ***

    This component is now ShareWare, even though it started as Freeware.  Earlier
versions (2.10 and before) are probably still floating around somewhere on the
web - so if the $10.00 registration fee is too steep - just use an older version.
It started as Freeware - but I have had so many requests for changes and enhancements
that it is starting to take up a lot of my time.  So in order to try to make up for
the time spent on this component - I am now asking $10.00 for full registration.  This
registration will allow you full support via Email and Phone.  It will also in-title
you to free upgrades whenever they are published. (ie - not just the next upgrade - but
all future upgrades as well!)

    If you make any improvements or would like to see an improvement to this component
please let me know!  I would like to hear any comments on this component (good & bad)
So let me know what you think!


  Notice:  This component is based on code from our File Manager II product.
           File Manager II (FM2) is a powerful file manager - that has some
           very useful tools added.  A File Scanning facility that allows you
           to do everything this component does.  A directory sizing utility -
           great for LAN managers.  It will show you the size of a directory
           (including the sizes of all sub directories), plus the size of each
           of the directories immedidately below the chosen directory - all in
           one action.  It also includes a directory compare utility - and others.

           If you are interested in this product you can find it on
               WWW.Winsite.Com   - Great Windows Software Repository
               FTP.Winsite.Com
               and Soon on
               WWW.Delphi32.Com

           It is called FM2_###.ZIP - where ### is the version number. (Current Ver 2.16)
           It is written in Delphi and it is a Shareware product (Cost: $25.00)
           Full source is available for another $50.00.

           Well - Enough of the sales pitch - on with the show ...


	See the end of this file for some BASM code sent to me that show
	a faster way to implement the file scan. I have not implemented this BASM
        code only because my knowledge of BASM is extremely limited. (Ok - Non-Existant!)
        I want to thank the person who sent it - as others may now benefit from it.

{------------------------------------------------------------------------------}


{------------------------------------------------------------------------------}
{  Component Properties Explanation                                            }
{------------------------------------------------------------------------------}
{
 Property CaseSense    : Determines whether the search will be case sensitive
 Boolean                   or not.

 Property AppendItems  : Determines whether the new file scan request will clear
 Boolean                   the existing items (remaining from prev search) or will
                           simply append the newly found items to results of last
                           search.
 Property CurrInfoLabel: This is the component that will receive the current search
 TLabel                    information text string.  It is used to hold the
                           'Now Searching XXX' string.

 Property SearchAllDrives: Determines if the search will incompass all available
 Boolean                     drives currently available to the user.  If it is
                             set to true - then it searches all drives.  If it is
                             false - then it lets the search take place in the
                             directory specified by the Directory propery.

 Property Interrupted  : Determines if the search should be interrupted.  Before
 Boolean                     scanning each file, the component processess application
                             messages.  This allows the developer to check to see if
                             the user wants to terminate the search (ie hits the escape
                             key or something similar).  All the programmer must to is
                             set the Interrupted property to True and the search will
                             terminate at that point.

 Property Directory    : This is the directory where the search will start.
 String                    This must be set for the search to procede correctly
                           The default is the root directory of C drive (C:)

 Property FileAge      : This is an Integer value of how old the files to be included
 Integer                   in the search are allowed to be.  File age is determined
                           by how old the file is compared to todays date.  For
                           example a file created today would be 0 days old.

 Property FileSpecs    : This is an String value of what file specs (ie. *.*, *.txt)
                           you would like included in the search.

 Property FileAgeComp  : This property is used to determine if the file age
TFileCompType              comparison will be =, >, <, >=, or <=.  If this value
(Defined below)            is fctNone - then no file age comparison is used.

 Property FileType     : The File type property is used to include the different
 TFileType                 types of files.  For example:  Hidden, Read Only,
 (Defined below)           System, Arhive, etc.

 Property FileSize     : This is an Integer value of how large the files to be included
 Integer                   in the search are allowed to be.  File size is counted
                           bytes, so a 300kb file is 300,000 bytes.

 Property FileSizeComp : This property is used to determine if the file size
 TFileCompType             comparison will be =, >, <, >=, or <=.  If this value
 (Defined below)           is fctNone - then no file size comparison is used.

 Property IncSubDir    : This property is used to determine if the search path
 Boolean                   will include all subdirectories under the path
                           specified in the Directory Property.

 Property SearchString : This property contains the search string that will be
 String                    used to scan the files for.  If this property is
                           blank then no search for the string is performed

 Property StartScan    : This property starts the search.  When set to True it
 Boolean                   starts the file find.  It is automatically set to
                           false by the termination of the search.
}
{------------------------------------------------------------------------------}
unit Tscnfile;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, stdctrls;

type
{------------------------------------------------------------------------}
{  Define the File type variable                                         }
{------------------------------------------------------------------------}
  TFileAttr = (ftReadOnly, ftHidden, ftSystem, ftVolumeID, ftArchive, ftNormal);
  TFileType = set of TFileAttr;

{------------------------------------------------------------------------}
{  Define the File type variable                                         }
{     Set the Properties FileAgeComp and/or FileSizeComp to any below    }
{  Except or fctNone - in order to include that type of comparison in the}
{  file scan.  fctNone - does not perform that type of compare           }
{------------------------------------------------------------------------}
  TFileCompType = (fctNone,fctEqual,fctGreaterThan,fctLessThan,fctGreaterThanOrEqual,fctLessThanOrEqual);

{------------------------------------------------------------------------}
{  Define the TfileScan Class                                            }
{------------------------------------------------------------------------}
  TFileScan = class(TComponent)
  private
    { Private declarations }
    FDirectory             :String;
    FSearchAllDrives       :Boolean;
    FIncFileSpecs          :String;
    FSearchString          :String;
    FIncFileType           :TFileType;
    FIncSubDir             :Boolean;
    FCaseSense             :Boolean;
    FFileSize              :LongInt;
    FFileSizeComp          :TFileCompType;
    FFileAge               :LongInt;
    FFileAgeComp           :TFileCompType;
    FStartScan             :Boolean;

    FShowSize              :Boolean;
    FShowDate              :Boolean;
    FShowAttr              :Boolean;

    FAppendItems           :Boolean;
    FCurrInfoLabel         :TLabel;

    FInterrupted           :Boolean;
    FScanResult            :TStringList;

  protected
    { Protected declarations }
    function ScanFile(FileName: String):Integer;
    Procedure LocateFiles(TreeBranch :String);
    Procedure SearchAllAvailDrives;
    Procedure SetDirectory(NewDirectory :String);
    function  GetIncFileType: TFileType;
    Procedure SetIncFileType(NewIncFileType: TFileType);
    Procedure SetCurrInfoLabel(NewLabel :TLabel);
    Procedure SetStartScan(NewValue :Boolean);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    Destructor Destroy;
    property Interrupted   : Boolean         read FInterrupted  write FInterrupted default False;
    property ScanResult    : TStringList     read FScanResult   write FScanResult;

  published
    { Published declarations }
    property FileType        : TFileType     read GetIncFileType   write SetIncFileType   default [ftNormal];
    property IncSubDir       : Boolean       read FIncSubDir       write FIncSubDir       default True;
    property Directory       : String        read FDirectory       write SetDirectory;
    property SearchAllDrives : Boolean       read FSearchAllDrives write FSearchAllDrives default False;
    property FileSpecs       : String        read FIncFileSpecs    write FIncFileSpecs;
    property SearchString    : String        read FSearchString    write FSearchString;
    property CaseSense       : Boolean       read FCaseSense       write FCaseSense       default False;
    property FileSize        : LongInt       read FFileSize        write FFileSize;
    Property FileSizeComp    : TFileCompType read FFileSizeComp    write FFileSizecomp    default fctNone;
    property FileAge         : LongInt       read FFileAge         write FFileAge;
    Property FileAgeComp     : TFileCompType read FFileAgeComp     write FFileAgecomp     default fctNone;
    Property CurrInfoLabel   : TLabel        read FCurrInfoLabel   write SetCurrInfoLabel;
    property StartScan       : Boolean       read FStartScan       write SetStartScan     default False;

    property ShowSize        : Boolean       read FShowSize        write FShowSize        default False;
    property ShowDate        : Boolean       read FShowDate        write FShowDate        default False;
    property ShowAttr        : Boolean       read FShowAttr        write FShowAttr        default False;
    property AppendItems     : Boolean       read FAppendItems     write FAppendItems     default False;
  end;

procedure Register;

{------------------------------------------------------------------------}
{------------------------------------------------------------------------}
{               S T A R T   O F  I M P L E M E N T A T I O N             }
{------------------------------------------------------------------------}
{------------------------------------------------------------------------}
implementation


{ Global Variable Declaration }
{ I know many people frown on using Global Variables - but in this case }
{  they are useful.  These variables are used in the recursive routine  }
{  LOCATEFILES.  If they were not global - then each directory searched }
{  would chew up more memory then it has to.                            }
Var
  FileDateAge, TodayAge                                        :Double;
  Year, Month, Day, Hour, Minute, Second, MSec                 :Word;
  DateStr, TimeStr                                             :String;
  TempFloat                                                    :Single;
  TempString                                                   :String;
  XReadOnly, XHidden, XSystem ,XVolumeId, XDirectory, XArchive :String;

{ -----------------------------------------------------------------------}
{  Register - Registers the component in the delphi component library    }
{     Change the 'Freeware' to put it in any panel you want!             }
{ -----------------------------------------------------------------------}
procedure Register;
begin
  RegisterComponents('Shareware', [TFileScan]);
end;

{ -----------------------------------------------------------------------}
{  Constructor                                                           }
{     Called everytime the component is created                          }
{ -----------------------------------------------------------------------}
constructor TFileScan.Create(AOwner: TComponent);
Var
   i        :Integer;
begin
  inherited Create(AOwner);
  FDirectory       := 'C:\';
  FSearchString    := '';
  FIncFileSpecs    := '*.*';
  FIncFileType     := [ftNormal];
  FIncSubDir       := True;
  FFileAge         := 0;
  FFileSize        := 0;
  FStartScan       := False;
  FShowSize        := False;
  FShowDate        := False;
  FShowAttr        := False;
  FAppendItems     := False;
  FInterrupted     := False;
  FSearchAllDrives := False;
  FScanResult      := TStringList.Create;
  FScanResult.Clear;

{  Now load all the TLabels on the form into the CurrInfoLabel Property }
  if (AOwner is TForm) then
  begin
     For I:=0 to TForm(AOwner).ComponentCount - 1 do
     begin
         if (TForm(AOwner).Components[I] is TLabel) then
            FCurrInfoLabel :=  TLabel(TForm(AOwner).Components[I]);
     end;
  end;


end;

{ -----------------------------------------------------------------------}
{  Destructor                                                            }
{     Called everytime the component is Destroyed                        }
{ -----------------------------------------------------------------------}
Destructor TFileScan.Destroy;
begin
  FScanResult.Free;  { Gotta Free the memory - or app will have memory leak!}
  inherited Destroy;
End;

{ -----------------------------------------------------------------------}
{  GetFileType                                                           }
{     This function is called to read the file type private variable     }
{ -----------------------------------------------------------------------}
Function TFileScan.GetIncFileType: TFileType;
begin
  Result := FIncFileType;
end;

{ -----------------------------------------------------------------------}
{  SetFileType                                                           }
{     This function is called to Write the file type private variable    }
{ -----------------------------------------------------------------------}
procedure TFileScan.SetIncFileType(NewIncFileType: TFileType);
begin
  if NewIncFileType <> FIncFileType then begin
    FIncFileType := NewIncFileType;
  end;
end;

{ -----------------------------------------------------------------------}
{  SetDirectory                                                          }
{     This function is called to read the Directory private variable     }
{ -----------------------------------------------------------------------}
Procedure TFileScan.SetDirectory(NewDirectory :String);
begin
   FDirectory := NewDirectory;
end;

{ -----------------------------------------------------------------------}
{  SetDirectory                                                          }
{     This function is called to read the Directory private variable     }
{ -----------------------------------------------------------------------}
Procedure TFileScan.SetStartScan(NewValue :Boolean);
Var
   SaveAppendItems                        :Boolean;
begin
   If NewValue then Begin
      { Set FStartScan to False - So that it can be used again by changing it to True}
      FStartScan   := False;
      FInterrupted := False;
      { Clear the String List for the new search (If AppendItems is not True) }
      If Not FAppendItems Then Begin
        FScanResult.Clear;
      End;
      If SearchAllDrives then begin
        { If Search AllDrives is set to true - then scan all Avail Drives}
        SaveAppendItems := FAppendItems;
        FAppendItems    := True;
        SearchAllAvailDrives;
        FAppendItems    := SaveAppendItems;
      End Else Begin
        LocateFiles(FDirectory);
      End;
   End;
end;

{ -----------------------------------------------------------------------}
{  SeachAllAvailDrives                                                   }
{     This procedure determines which drives the user currently has      }
{ access to and then scans them                                          }
{ -----------------------------------------------------------------------}
Procedure TFileScan.SearchAllAvailDrives;
Var
   Drive             :Integer;
   DriveString       :String;
Begin
   For Drive := 0 to 25 Do Begin  {Scan all 26 drives - 'A' to 'Z' }
     DriveString := Chr(Drive + Ord('a'));
     If GetDriveType(Drive) <> 0 then Begin
       DriveString := Chr(Drive + Ord('a')) + ':\';
       LocateFiles(DriveString);
     End; { If GetDriveType }
   End;
End;


{ -----------------------------------------------------------------------}
{  Locate Files                                                          }
{     This function looks for files that match the search criteria       }
{ -----------------------------------------------------------------------}
Procedure TFileScan.LocateFiles(TreeBranch :String);
var
  FSearchRec, DSearchRec                                       :TSearchRec;
  FindResult                                                   :Integer;
  FileAttr                                                     :Byte;
  ScanResult                                                   :Integer;
  AbortSearch                                                  :Boolean;
  FoundMatch                                                   :Boolean;
  i                                                            :Integer;

Begin
  AbortSearch      := False;
  FoundMatch       := False;

  { Setup the File Age and Size Variables }
  TodayAge         := Date + 0.99999;

{ Make Sure that the Directory Branch ends with a '\' }
  If TreeBranch[Length(TreeBranch)] <> '\' Then Begin
     TreeBranch := TreeBranch + '\';
  End;

{ Find All File Types that have been selected }
  FileAttr := 0;
  If ftReadOnly in FIncFileType then
    FileAttr := FileAttr + faReadOnly;
  If ftHidden in FIncFileType then
    FileAttr := FileAttr + faHidden;
  If ftSystem in FIncFileType then
    FileAttr := FileAttr + faSysFile;
  If ftVolumeId in FIncFileType then
    FileAttr := FileAttr + faVolumeId;
  If ftArchive in FIncFileType then
    FileAttr := FileAttr + faArchive;

{--------------------------------------------------------------------------}
{  Now Start the Actual Search for files that match the criteria           }
{--------------------------------------------------------------------------}

  FindResult := FindFirst(TreeBranch + FIncFileSpecs ,FileAttr, FSearchRec);

  Try
    While ((FindResult=0) and Not (FInterrupted)) Do Begin { Now loop through all the files that match }
{ Update the current Search Information for the CurrInfoComp }
      If CurrInfoLabel <> Nil then begin
         { Make sure to clear the label area - remember spaces take up very little space!!}
         CurrInfoLabel.Caption := TreeBranch + FSearchRec.Name +
                                  '                          ' +
                                  '                          ' +
                                  '                          ';
         CurrInfoLabel.Refresh;
      End;

      FoundMatch  := True;
      AbortSearch := False;
      {---------------------------------------------------------------------}
      { Now check to see if the search should contain a check for file size }
      {---------------------------------------------------------------------}
      If Not AbortSearch Then Begin
      If FileSizeComp <> fctNone then Begin
        Case FileSizeComp of
          fctEqual:
            If Not (FSearchRec.Size = FileSize) then begin
              FoundMatch  := False;
              AbortSearch := True;
            End;
          fctGreaterThan:
            If Not (FSearchRec.Size > FileSize) then begin
              FoundMatch  := False;
              AbortSearch := True;
            End;
          fctGreaterThanOrEqual:
            If Not (FSearchRec.Size >= FileSize) then begin
              FoundMatch  := False;
              AbortSearch := True;
            End;
          fctLessThan:
            If Not (FSearchRec.Size < FileSize) then begin
              FoundMatch  := False;
              AbortSearch := True;
            End;
          fctLessThanOrEqual:
            If Not (FSearchRec.Size <= FileSize) then begin
              FoundMatch  := False;
              AbortSearch := True;
            End;
        End;
      End;
      End;

      {---------------------------------------------------------------------}
      { Now check the File Age Limit                                        }
      {---------------------------------------------------------------------}
      If Not AbortSearch then Begin
      If (FileAgeComp <> fctNone) then begin
        FileDateAge := TodayAge - FileDatetoDateTime(FSearchRec.Time);
        Case FileAgeComp of
          fctEqual:
            If Not (Trunc(FileDateAge) = FileAge) then begin
              FoundMatch  := False;
              AbortSearch := True;
            End;
          fctGreaterThan:
            If Not (Trunc(FileDateAge) > FileAge) then begin
              FoundMatch  := False;
              AbortSearch := True;
            End;
          fctGreaterThanOrEqual:
            If Not (Trunc(FileDateAge) >= FileAge) then begin
              FoundMatch  := False;
              AbortSearch := True;
            End;
          fctLessThan:
            If Not (Trunc(FileDateAge) < FileAge) then begin
              FoundMatch  := False;
              AbortSearch := True;
            End;
          fctLessThanOrEqual:
            If Not (Trunc(FileDateAge) <= FileAge) then begin
              FoundMatch  := False;
              AbortSearch := True;
            End;
        End;
      End;
      End;

      {---------------------------------------------------------------------}
      { Now check to see if the file contains the search string in it       }
      {---------------------------------------------------------------------}
      If Not AbortSearch Then Begin
        If (SearchString <> '') Then Begin
          ScanResult := ScanFile(TreeBranch + FSearchRec.Name);
          IF ScanResult <= 0 then Begin
            FoundMatch  := False;         { If zero or Lower - no match found          }
            If ScanResult < 0 then begin  { If Less than Zero - then error during scan }
              AbortSearch := True;
              FScanResult.Add('*** Error Scanning: ' + TreeBranch + FSearchRec.Name);
            End;
          End;
        End;
      End;

      {---------------------------------------------------------------------}
      { Now - If a Match has be Found - then add it to the output stringlist}
      {---------------------------------------------------------------------}
      If FoundMatch then begin

        TempString := TreeBranch + FSearchRec.Name;
        IF FShowSize Then Begin
          TempFloat  := FSearchRec.Size;
          TempString := TempString + '  ' + Format('%12.0n',[TempFloat]);
        End;

        IF FShowDate Then Begin
          DecodeDate(FileDatetoDateTime(FSearchRec.Time), Year, Month, Day);
          DecodeTime(FileDatetoDateTime(FSearchRec.Time), Hour, Minute, Second, MSec);
          DateStr    := Format('%4.2d',[Year]) + '/' + Format('%2.2d',[Month]) + '/' + Format('%2.2d',[Day]);
          TimeStr    := Format('%2.2d',[Hour]) + ':' + Format('%2.2d',[Minute]) + ':' + Format('%2.2d',[Second]);
          TempString := TempString + '  ' +  Format('%10s',[DateStr]) + '  ' + Format('%8s',[TimeStr]);
        End;
{--------------------------------------------------}
        If FShowAttr Then Begin
          IF (faReadOnly AND FSearchRec.Attr) = faReadOnly then Begin
            XReadOnly := 'R';
          End Else Begin
            XReadOnly := '-';
          End;
          IF (faHidden AND FSearchRec.Attr) = faHidden then Begin
            XHidden := 'H';
          End Else Begin
            XHidden := '-';
          End;
          IF (faSysFile AND FSearchRec.Attr) = faSysFile then Begin
            XSystem := 'S';
          End Else Begin
            XSystem := '-';
          End;
          IF (faVolumeId AND FSearchRec.Attr) = FaVolumeId then Begin
            XVolumeId := 'V';
          End Else Begin
            XVolumeId := '-';
          End;
          IF (faDirectory AND FSearchRec.Attr) = faDirectory then Begin
            XDirectory := 'D';
          End Else Begin
            XDirectory := '-';
          End;
          IF (faArchive AND FSearchRec.Attr) = faArchive then Begin
            XArchive := 'A';
          End Else Begin
             XArchive := '-';
          End;
          TempString := TempString  +  '  ' +
                                    XReadOnly  +
                                    XHidden    +
                                    XSystem    +
                                    XVolumeId  +
                                    XDirectory +
                                    XArchive;
        End;
{--------------------------------------------------}
        FScanResult.Add(TempString);
      End;

 { Now Check to see if the user wishes to interrupt the Search
   Remember - the interrupt handling code must be in your application - it is
              note in this component.  Your code - must set the Interrupted
              Property for this to work!
   Example of Code  - Call from KeyPress for the Form ??
             Procedure CheckforInterrupt(Key :Char);
             Begin
               If Key = Chr(VK_ESCAPE) then begin
                 TFileScan1.Interrupted := True;
               End;
             End;
 }
        Application.ProcessMessages;
        If FInterrupted Then Begin
          FoundMatch := False;
          AbortSearch := True;
          FScanResult.Add('ZZ*** Search Aborted by User ***ZZ');
          FScanResult.Add('AA*** Search Aborted by User ***AA');
          Exit;
        End;

      FindResult  := FindNext(FSearchRec); {get the next file that matches}
    End;

    {---------------------------------------------------------------------}
    { Now - Search sub directories if that option has been selected       }
    {---------------------------------------------------------------------}
    If  FIncSubDir then Begin
       FindResult := FindFirst(TreeBranch + '*.*', faDirectory, DSearchRec);

       While ((FindResult=0) and Not (FInterrupted)) Do Begin
         If ((DSearchRec.Attr and faDirectory) = faDirectory)  and (pos('.', DSearchRec.Name) = 0) then begin
           LocateFiles(TreeBranch + DSearchRec.Name);
         End;
         FindResult := FindNext(DSearchRec);
       End;
    End;
  Finally
    SysUtils.FindClose(FSearchRec);
    SysUtils.FindClose(DSearchRec);
  End;
End;



{ ----------------------------------------------------------------------------}
{  ScanFile                                                                   }
{     This function is the guts of File Scan code                             }
{     It performs the actual file scanning.  Note that this code deals with   }
{  files that have null bytes embedded in the file (ie executables, word      }
{  processing documents, etc.)                                                }
{                                                                             }
{ ----------------------------------------------------------------------------}
Function TFileScan.ScanFile(FileName: String):Integer;
Const
   FileError = -1;
var
   FileHandle               :Integer; { Handle of the file to be scanned }
   ReadCount                :Integer;
   i                        :Integer;
   FileNamePChar            :PChar;
   SearchStringPChar        :PChar;
   FileBuffer               :PChar;
   StringPosPChar           :PChar;
   SearchPchar              :PChar;
   BufferSize               :Word;
   ReadResult               :Word;

Begin
{ Initialize appropriate Pointers and setup null terminated strings }
     Result   := 0;
     BufferSize := 30000;   { Read the file in 30,000 byte chunks }
     { Allocate Memory for Null terminated File name }

     GetMem(FileBuffer,BufferSize + 2);  {allocate more to account for the null bytes to be added}
     GetMem(SearchStringPChar,Sizeof(FSearchString));
     StrPCopy(SearchStringPChar, FSearchString);

{$IFDEF WIN32}
   FileNamePchar := PChar(FileName);
{$ELSE}
   Getmem(FileNamePChar,Sizeof(FileName));
   FileNamePchar := StrPCopy(FileNamePChar,FileName);
{$ENDIF}

     Try
       FileHandle := _lopen(FileNamePChar,of_Read);
       { If there is any error opening file - then terminate search }
       If FileHandle = FileError Then Begin
          Result := -1;
       End Else Begin
       { Scan through the file - terminate search if string found }
         Try
          ReadCount  := 1;
          Result     := 0;
          { Read the first 30,000 byte chunk of the file }
          ReadResult := _lread(FileHandle, FileBuffer, BufferSize);
          { Now add the null byte to the end of the chunch that was read from the file }

          SearchPchar := FileBuffer + ReadResult + 1;
          SearchPChar := StrPCopy(SearchPChar,Chr(0));

          While ReadResult > 0 do Begin
             { Assign another pointer to the string - so that we can change it }
             SearchPchar := FileBuffer;
             i           := 0;
             { Scan until i is greater than the number of bytes read from the file }
             while I < ReadResult do begin
               Try
                 If StrLen(SearchPChar) > 0 then begin
                   If FCaseSense then Begin
                     StringPosPChar   := StrPos(SearchPChar,SearchStringPChar);
                   End Else Begin
                     StringPosPChar   := StrPos(StrUpper(SearchPChar),StrUpper(SearchStringPChar));
                   End;
                   If StringPosPChar <> nil then begin
                     Result         := 1;
                     StringPosPChar := Nil;
                     SearchPChar    := Nil;
                     Break;  { Exit the While loop - since you found the string }
                   end else begin
                     Result         := 0;
                   End;
                 End;
                 i                  := i + StrLen(SearchPChar) + 1;
                 SearchPchar        := SearchPChar + StrLen(SearchPChar) + 1;
                 If i >= ReadResult then Begin
                    Break;
                 End;
               Except
                 StringPosPChar := Nil;
                 SearchPChar    := Nil;
                 i              := ReadResult + 1;
               End;
             End;

            { If String has been found - then exit read loop }
            If Result > 0 then begin
               ReadResult := 0; {Terminate the read loop }
               Break;
            End;

            ReadResult := _lread(FileHandle, FileBuffer, BufferSize);
            { Now add the null byte to the end of the chunch that was read from the file }
            SearchPchar := FileBuffer + ReadResult + 1;
            SearchPChar := Chr(0);
            Inc(ReadCount);  {Used for debugging - I just left it in}
          End;  { While ReadResult > 0 }

          { If there was some error (other than end of file) - then return a negative}
          If ReadResult < 0 then begin
            Result := -1;
          End;
         Except
           StringPosPChar := Nil;
           Result := -2;
         End; { to Try - Except }
       End;  { IF FileHandle = FileError }

     Finally
     { Dispose of all outstanding variables - and close the file }
       FreeMem(SearchStringPChar,Length(FSearchString));
       FreeMem(FileBuffer,BufferSize + 2);

{$IFNDEF WIN32}
   FreeMem(FileNamePChar,Length(FileName));
{$ENDIF}

       _lclose(FileHandle);
     End; { Try - Finally }
End;


{ -------------------------------------------------------------------------}
{  SetCurrInfoLabel                                                        }
{     This function is called to write the FCurrInfoLabel private variable }
{ -------------------------------------------------------------------------}
Procedure TFileScan.SetCurrInfoLabel(NewLabel :TLabel);
begin
   FCurrInfoLabel := NewLabel;
end;


{ -----------------------------------------------------------------------
 This next routine (DirExists) - I have included for your use - I have
 found it useful in many program that deal with the file system - but it
 is not part of this component.
{ -----------------------------------------------------------------------}

{ -----------------------------------------------------------------------}
{  DirExists
     This function determines whether a directory exists or not.
     It is a modified version of DirectoryExists.  Borlands version had
     a bug in it - it did not consider 'C:' to be a directory.  I consider
     any valid drive to be a directory - it is the root directory....
{ -----------------------------------------------------------------------}
{function DirExists(Name: string): Boolean;
var
  SearchRec  :TSearchRec;
  Drive      :Integer;
begin
  if Name[Length(Name)] = '\' then Dec(Name[0]);
  if (Length(Name) = 2) and (Name[2] = ':') then begin
    Name := UpperCase(Name);
    Drive := Ord(Name[1]) - Ord('A');
    If GetDriveType(Drive) > 0 then begin
      Result := True;
    End Else Begin
      Result := False;
    End;
  End else Begin
    Result := FindFirst(Name, faDirectory, SearchRec) = 0;
    If Result then begin
      Result := Result and (SearchRec.Attr and faDirectory <> 0);
    End;
  End;
end;
}

end.
